from dataclasses import dataclass

import conversationalspacemapapp.Types.Constants as Constants


@dataclass
class ParticipantOptions:
    name: str
    type: Constants.Participant
    label: str
    color: str


@dataclass
class PlotOptions:
    def __init__(
        self,
        app,
        title="Conversational Map Space",
        show_title=True,
        labels=True,
        interviewer_label="Interviewer",
        interviewee_label="Interviewee",
        yaxis=True,
        xaxis=True,
        grid=True,
        legend=True,
    ):
        assert app.has_parser
        self.map = app.parser.map
        self.participants: list[ParticipantOptions] = self._get_participants(app)

        self.title = title
        self.show_title = show_title
        self.labels = labels
        self.interviewer_label = interviewer_label
        self.interviewee_label = interviewee_label
        self.yaxis = yaxis
        self.xaxis = xaxis
        self.grid = grid
        self.legend = legend

    def _get_participant(self, participant_name: str):
        for participant in self.participants:
            if participant.name == participant_name:
                return participant

    def get_participant_color(self, participant_name: str) -> str:
        return self._get_participant(participant_name).color

    def get_participant_label(self, participant_name: str) -> str:
        return self._get_participant(participant_name).label

    def get_participant_type(self, participant_name: str) -> Constants.Participant:
        return self._get_participant(participant_name).type

    @staticmethod
    def _get_participants(app) -> list[ParticipantOptions]:
        participants = app.parser.participants
        output = []
        for participant in participants:
            output.append(
                ParticipantOptions(
                    name=participant,
                    type=app._get_participant_role(participant),
                    label=app._get_participant_name(participant),
                    color=app._get_participant_color(participant),
                )
            )
        return output


@dataclass
class Utterance:
    number: int
    speaker: str
    words: int

    def __lt__(self, other):
        return self.number < other.number



import unittest

import conversationalspacemapapp.Plotter.StylePicker as ColorPicker


class TestColorPicker(unittest.TestCase):
    def abstract_test(self, palette, palette_str, pallette_list):
        self.assertEqual(palette.value, pallette_list)
        self.assertEqual(str(palette), palette_str)

    def test_accent(self):
        self.abstract_test(
            ColorPicker.Palette.Accent,
            "Accent",
            [
                "#7fc97f",
                "#beaed4",
                "#fdc086",
                "#ffff99",
                "#386cb0",
                "#f0027f",
                "#bf5b17",
                "#666666",
            ],
        )

    def test_pastel(self):
        self.abstract_test(
            ColorPicker.Palette.Pastel,
            "Pastel",
            [
                "#b3e2cd",
                "#fdcdac",
                "#cbd5e8",
                "#f4cae4",
                "#e6f5c9",
                "#fff2ae",
                "#f1e2cc",
                "#cccccc",
            ],
        )

    def test_set(self):
        self.abstract_test(
            ColorPicker.Palette.Set,
            "Set",
            [
                "#66c2a5",
                "#fc8d62",
                "#8da0cb",
                "#e78ac3",
                "#a6d854",
                "#ffd92f",
                "#e5c494",
                "#b3b3b3",
            ],
        )

    def test_tab(self):
        self.abstract_test(
            ColorPicker.Palette.Tab,
            "Tab",
            [
                "#1f77b4",
                "#aec7e8",
                "#ff7f0e",
                "#ffbb78",
                "#2ca02c",
                "#98df8a",
                "#d62728",
                "#ff9896",
            ],
        )



from enum import Enum


class Participant(Enum):
    Interviewer = -1
    Interviewee = 1
    Undefined = 0

    @property
    def constant(self):
        return self.value

    def __str__(self):
        if self.value == Participant.Interviewee.value:
            return "Interviewee"
        elif self.value == Participant.Interviewer.value:
            return "Interviewer"
        else:
            return "Undefined"



import toga
import toga_chart
from toga.style import Pack
from toga.constants import COLUMN

import pathlib
import platform
from typing import Callable

import conversationalspacemapapp.Types.Data as Data
import conversationalspacemapapp.Plotter.PlotMap as PlotMap
import conversationalspacemapapp.Types.Constants as Constants
import conversationalspacemapapp.App.AbstractApp as AbstractApp
import conversationalspacemapapp.Plotter.StylePicker as StylePicker


class ConversationalSpaceMapAppToga(AbstractApp.AbstractApp, toga.App):
    default_padding = 5
    default_flex = 1

    def __init__(self, name, id):
        super(ConversationalSpaceMapAppToga, self).__init__(formal_name=name, app_id=id)

    @property
    def path(self) -> pathlib.Path | None:
        return self.path_input.value

    @property
    def has_path(self):
        return self.path is not None and self.path.is_file()

    @property
    def path_filename(self) -> str | None:
        return str(self.path.name) if self.has_path else None

    @property
    def has_parser(self):
        return self.parser is not None

    def startup(self):
        self._create_window()

    def _set_window(self, tab_menu):
        self.main_window = toga.MainWindow()
        self.main_window.content = tab_menu
        self.main_window.size = toga.Size(width=1300, height=1000)
        self.main_window.show()

    def _create_tab_menu(self, tabs):
        main = toga.OptionContainer(content=tabs)
        self._set_widget_style(main)
        self._set_transparent_background(main)
        return main

    def _create_about_layout(self):
        # Create app description page
        description = toga.WebView(
            url="https://manuelbieri.ch/ConversationalSpaceMapApp/"
        )
        self._set_widget_style(description)
        self._set_transparent_background(description)
        about = toga.Box(
            children=[description],
            style=Pack(direction=COLUMN),
        )
        return about

    def _create_transcript_layout(self):
        self.transcript = toga.MultilineTextInput()
        self._set_widget_style(self.transcript)
        if self.has_parser:
            self.transcript.value = self.parser.content
        else:
            self.transcript.enabled = False
        return self.transcript

    def _set_transcript(self, content: str):
        assert type(content) == str
        self.transcript.value = content

    def _set_home_window(self, plot_settings, participants, label, chart):
        assert plot_settings is not None
        assert participants is not None
        assert label is not None
        assert chart is not None
        return toga.Box(
            children=[plot_settings, participants, label, chart],
            style=Pack(direction=COLUMN),
        )

    def _create_plot_settings_layout(self):
        # Create selections
        self.file_format = toga.Selection(
            items=ConversationalSpaceMapAppToga.save_file_formats
        )
        self._set_widget_style(self.file_format, flex=0)
        self.path_input = toga.Selection(
            items=self._get_file_history(), on_change=self._set_parser
        )
        self._set_widget_style(self.path_input, flex=1)
        self.path_input.readonly = True

        # Create buttons
        self.button = self._button_factory("📄", on_press=self.open_handler)
        self._set_widget_style(self.button, flex=0)
        self.plot = self._button_factory(
            "🖌", on_press=self.plot_handler, enabled=self.has_path
        )
        self._set_widget_style(self.plot, flex=0)
        self.save = self._button_factory(
            "💾", on_press=self.save_handler, enabled=False
        )
        self._set_widget_style(self.save, flex=0)
        plot_settings = toga.Box(
            children=[
                self.path_input,
                self.button,
                self.plot,
                self.file_format,
                self.save,
            ]
        )
        return plot_settings

    def _create_initial_participants_layout(self):
        self.participants_layout = toga.OptionContainer(
            content=[("General", self._create_general_participants_layout())],
            style=Pack(
                padding=ConversationalSpaceMapAppToga.default_padding, height=60
            ),
        )
        return self.participants_layout

    def _create_general_participants_layout(self):
        self.color_palette = self._set_widget_style(
            toga.Selection(
                items=StylePicker.Palette.available_palettes(),
                on_change=self._set_participants_color_selection,
            ),
        )
        self.plot_title = self._set_widget_style(
            toga.Switch(text="Title", on_change=self.plot_handler, value=True), flex=0
        )
        self.plot_title_input = self._set_widget_style(
            toga.TextInput(placeholder="Title"), flex=2
        )
        self.interviewer_label_input = self._set_widget_style(
            toga.TextInput(
                value="Interviewer",
                placeholder="Interviewer label",
                on_change=self.plot_handler,
            )
        )
        self.interviewee_label_input = self._set_widget_style(
            toga.TextInput(
                value="Interviewee",
                placeholder="Interviewee label",
                on_change=self.plot_handler,
            )
        )
        self.plot_legend = self._set_widget_style(
            toga.Switch(text="Legend", on_change=self.plot_handler, value=True), flex=0
        )
        self.plot_labels = self._set_widget_style(
            toga.Switch(text="Labels", on_change=self.plot_handler, value=True), flex=0
        )
        self.plot_yaxis = self._set_widget_style(
            toga.Switch(text="Y-Axis", on_change=self.plot_handler, value=True), flex=0
        )
        self.plot_xaxis = self._set_widget_style(
            toga.Switch(text="X-Axis", on_change=self.plot_handler, value=True), flex=0
        )
        self.plot_grid = self._set_widget_style(
            toga.Switch(text="Grid", on_change=self.plot_handler, value=True), flex=0
        )
        self._general_participants_layout = toga.Box(
            children=[
                self.plot_title_input,
                self.plot_title,
                self.plot_labels,
                self.interviewer_label_input,
                self.interviewee_label_input,
                self.plot_yaxis,
                self.plot_xaxis,
                self.plot_legend,
                self.plot_grid,
                self.color_palette,
            ]
        )
        return self._general_participants_layout

    def _create_info_layout(self):
        self.label = toga.Label("")
        self._set_widget_style(self.label)
        return self.label

    def _set_info_layout(self):
        assert self.has_parser
        self.label.text = self._get_info_content()
        self.label.refresh()

    def _create_chart(self):
        self.chart = toga_chart.Chart(style=Pack(flex=1), on_draw=self.draw_chart)
        self._set_widget_style(self.chart)
        return self.chart

    def _create_participants_layout(self):
        assert self.has_path
        self._clear_participants_layout()
        for participant in self.parser.participants:
            self.participants_layout.content.append(
                participant, self._create_participant_layout(participant)
            )

    def _clear_participants_layout(self):
        self.participants_layout.current_tab = 0
        for _ in range(1, len(self.participants_layout.content)):
            self.participants_layout.content.remove(1)
        assert len(self.participants_layout.content) == 1

    def _create_participant_layout(self, participant: str) -> toga.Box:
        # Create participant role
        role = toga.Selection(
            items=Constants.Participant,
            id=participant + "_role",
            on_change=self.plot_handler,
        )
        self._set_widget_style(role)

        # Create participant name label
        name = toga.TextInput(
            value=participant,
            id=participant + "_name",
            on_change=self.plot_handler,
            placeholder="Speaker name",
        )
        self._set_widget_style(name)

        # Create color picker
        color = toga.TextInput(
            value=self.color_palette.value.value[0],
            id=participant + "_color",
            on_change=self.plot_handler,
            readonly=True,
        )
        self._set_widget_style(color, flex=0)

        widget_container = toga.Box(
            children=[
                role,
                name,
                color,
            ]
        )

        for i in range(8):
            button = self._set_widget_style(
                toga.Button(
                    "   ",
                    id=participant + "_color" + str(i),
                    style=Pack(background_color=self.color_palette.value.value[i]),
                    on_press=lambda e: self._set_participant_color(color, e),
                ),
                flex=0,
            )
            widget_container.add(button)

        return widget_container

    def _set_participants_color_selection(self, e):
        assert self.has_parser
        for participant in self.parser.participants:
            for i in range(8):
                color_button = self._get_widget_by_id(participant + "_color" + str(i))
                color_button.style.background_color = self.color_palette.value.value[i]

    def _set_participant_color(self, color_widget: toga.Widget, index) -> None:
        color_widget.value = StylePicker.ColorPicker.rgb2hex(
            index.style.background_color.r,
            index.style.background_color.g,
            index.style.background_color.b,
        )

    def draw_chart(self, chart: toga_chart.Chart, figure, *args, **kwargs):
        if self.has_parser:
            self.map = PlotMap.MapBarPlot(fig=figure)
            self.map.plot(
                options=Data.PlotOptions(
                    app=self,
                    title=self.plot_title_input.value,
                    show_title=self.plot_title.value,
                    labels=self.plot_labels.value,
                    interviewer_label=self.interviewer_label_input.value,
                    interviewee_label=self.interviewee_label_input.value,
                    yaxis=self.plot_yaxis.value,
                    xaxis=self.plot_xaxis.value,
                    legend=self.plot_legend.value,
                    grid=self.plot_grid.value,
                ),
            )
            figure.tight_layout()
        else:
            return

    def _button_factory(
        self,
        label: str,
        on_press: Callable,
        enabled: bool = True,
        padding: float = None,
        flex: float = None,
    ) -> toga.Button:
        button = toga.Button(label, on_press=on_press)
        button.enabled = enabled
        self._set_widget_style(button, padding=padding, flex=flex)
        return button

    @staticmethod
    def _set_default_widget_flex(widget: toga.Widget) -> toga.Widget:
        widget.style.flex = ConversationalSpaceMapAppToga.default_flex
        return widget

    @staticmethod
    def _set_default_widget_padding(widget: toga.Widget) -> toga.Widget:
        widget.style.padding = ConversationalSpaceMapAppToga.default_padding
        return widget

    @staticmethod
    def _set_widget_style(
        widget: toga.Widget,
        padding: float = None,
        flex: float = None,
    ) -> toga.Widget:
        if padding is None and flex is None:
            ConversationalSpaceMapAppToga._set_default_widget_flex(widget)
            ConversationalSpaceMapAppToga._set_default_widget_padding(widget)
        elif padding is None:
            ConversationalSpaceMapAppToga._set_default_widget_padding(widget)
            widget.style.flex = flex
        elif flex is None:
            ConversationalSpaceMapAppToga._set_default_widget_flex(widget)
            widget.style.padding = padding
        else:
            widget.style.padding = padding
            widget.style.flex = flex
        return widget

    @staticmethod
    def _set_transparent_background(widget):
        if "macOS" in platform.platform():
            widget.style.background_color = "transparent"

    async def _get_path(self):
        file = toga.OpenFileDialog("Open file", file_types=["txt"])
        return await self.main_window.dialog(file)

    def _set_path(self, path: pathlib.Path):
        assert path.is_file()
        try:
            self.path_input.value = path
        except ValueError:
            self.path_input.items.append(path)
            self.path_input.value = path
        self._set_parser()

    def _set_plot_title(self):
        assert self.has_path
        self.plot_title_input.value = "Conversational Space Map " + str(self.path.stem)

    def _update_plot(self):
        self.chart.redraw()
        self.save.enabled = True

    def _is_new_history_path(self) -> bool:
        if self.path in self._get_file_history():
            return False
        return True

    async def _get_save_path(self):
        assert self.has_path
        file = toga.SaveFileDialog(
            "Save file",
            suggested_filename=str(pathlib.Path(self.path).stem),
            file_types=ConversationalSpaceMapAppToga.save_file_formats,
        )
        return await self.main_window.dialog(file)

    def _get_widget_value_by_id(self, key: str, default_value=None):
        widget: toga.Widget = self._get_widget_by_id(key)
        if widget is None:
            return default_value
        return widget.value

    def _get_widget_by_id(self, key: str) -> toga.Widget | None:
        for tab in self.participants_layout.content:
            for child in tab.content.children:
                if child.id == key:
                    return child
        return None


def main():
    return ConversationalSpaceMapAppToga(
        "ConversationalSpaceMapApp", "ch.manuelbieri.conversationalspacemapapp"
    )



import conversationalspacemapapp.App.TogaApp.app as togaApp


def main(toga=True):
    if toga:
        togaApp.main().main_loop()
    else:
        NotImplemented("No other GUI implementation available.")


if __name__ == "__main__":
    main()



import unittest

import utilities

import conversationalspacemapapp.Types.Data as Data
import conversationalspacemapapp.App.AbstractApp as AbstractApp
import conversationalspacemapapp.Parser.TimestampParser as TranscriptParser


class TestPlotOptions(unittest.TestCase):
    parser: TranscriptParser.AbstractParser
    app: AbstractApp.AbstractApp

    def setUp(self):
        TestPlotOptions.parser = utilities.get_parser_mock()
        TestPlotOptions.app = utilities.get_app_mock(parser=TestPlotOptions.parser)

    def test_plot_options_setup(self):
        sut = Data.PlotOptions(
            app=TestPlotOptions.app,
        )
        speaker00 = sut.participants[0]
        speaker01 = sut.participants[1]
        self.assertEqual(2, len(sut.participants))
        self.assertEqual(utilities.speaker00_name, speaker00.name)
        self.assertEqual(utilities.speaker00_label, speaker00.label)
        self.assertEqual(utilities.speaker00_color, speaker00.color)
        self.assertEqual(utilities.speaker00_type, speaker00.type)
        self.assertEqual(utilities.speaker01_name, speaker01.name)
        self.assertEqual(utilities.speaker01_label, speaker01.label)
        self.assertEqual(utilities.speaker01_color, speaker01.color)
        self.assertEqual(utilities.speaker01_type, speaker01.type)



import pathlib


path_short_transcript = (
    pathlib.Path(__file__).parent / "test_short_transcript/transcription_timestamps.txt"
)
content_short_transcript = """Interviewer Ich habe noch nie so ein schlechtes Interview gesehen. Red jetzt nicht so klar, Mona.\n\nInterviewee Jetzt noch etwas nuscheln.\n\n"""
path_long_transcript = (
    pathlib.Path(__file__).parent / "test_transcript/transcription_timestamps.txt"
)
path_multiple_speaker_transcript = (
    pathlib.Path(__file__).parent
    / "test_transcript_multiple_speakers/transcription_timestamps.txt"
)



import unittest
import matplotlib
import matplotlib.pyplot as plt

matplotlib.use("Agg")

import utilities

import conversationalspacemapapp.Types.Data as Data
import conversationalspacemapapp.Plotter.PlotMap as PlotMap
import conversationalspacemapapp.App.AbstractApp as AbstractApp
import conversationalspacemapapp.Parser.TimestampParser as TranscriptParser


class TestPlotMap(unittest.TestCase):
    parser: TranscriptParser.AbstractParser
    fig: plt.figure()
    ax: plt.Axes
    app: AbstractApp.AbstractApp

    def setUp(self):
        TestPlotMap.parser = utilities.get_parser_mock()
        TestPlotMap.app = utilities.get_app_mock(parser=TestPlotMap.parser)
        TestPlotMap.fig, TestPlotMap.ax = plt.subplots()

    def tearDown(self):
        TestPlotMap.fig.show()

    def test_bar_plot(self):
        sut = PlotMap.MapBarPlot(
            fig=TestPlotMap.fig,
        )
        sut.plot(Data.PlotOptions(app=TestPlotMap.app))

    def test_bar_plot_no_axes(self):
        sut = PlotMap.MapBarPlot(
            fig=TestPlotMap.fig,
        )
        sut.plot(
            Data.PlotOptions(
                app=TestPlotMap.app,
                xaxis=False,
                yaxis=False,
            )
        )



from enum import Enum
import matplotlib as mpl
import matplotlib.colors as mcolors


class ColorPicker:
    COLORS = mcolors.CSS4_COLORS

    @staticmethod
    def rgb2hex(r, g, b):
        return "#" + "".join(f"{i:02X}" for i in (r, g, b))

    @staticmethod
    def pastel():
        return ColorPicker._get_colormap_list("Pastel2")

    @staticmethod
    def accent():
        return ColorPicker._get_colormap_list("Accent")

    @staticmethod
    def set():
        return ColorPicker._get_colormap_list("Set2")

    @staticmethod
    def tab():
        return ColorPicker._get_colormap_list("tab20")

    @staticmethod
    def _get_colormap_list(name: str) -> list:
        cmap = mpl.colormaps.get_cmap(name)
        return [mpl.colors.to_hex(cmap(i)) for i in range(8)]


class Palette(Enum):
    Pastel = ColorPicker.pastel()
    Accent = ColorPicker.accent()
    Set = ColorPicker.set()
    Tab = ColorPicker.tab()

    @staticmethod
    def available_palettes() -> list:
        return [Palette.Accent, Palette.Pastel, Palette.Set, Palette.Tab]

    def __str__(self):
        if self.value == Palette.Pastel.value:
            return "Pastel"
        elif self.value == Palette.Accent.value:
            return "Accent"
        elif self.value == Palette.Set.value:
            return "Set"
        else:
            return "Tab"



import unittest

import conversationalspacemapapp.Types.Constants as Constants


class TestCustomConstants(unittest.TestCase):
    def test_interviewer_value(self):
        self.assertEqual(Constants.Participant.Interviewer.constant, -1)

    def test_interviewee_value(self):
        self.assertEqual(Constants.Participant.Interviewee.constant, 1)

    def test_undefined_value(self):
        self.assertEqual(Constants.Participant.Undefined.constant, 0)

    def test_interviewer_name(self):
        self.assertEqual(Constants.Participant.Interviewer.name, "Interviewer")
        self.assertEqual(str(Constants.Participant.Interviewer), "Interviewer")

    def test_interviewee_name(self):
        self.assertEqual(Constants.Participant.Interviewee.name, "Interviewee")
        self.assertEqual(str(Constants.Participant.Interviewee), "Interviewee")

    def test_undefined_name(self):
        self.assertEqual(Constants.Participant.Undefined.name, "Undefined")
        self.assertEqual(str(Constants.Participant.Undefined), "Undefined")



import matplotlib.pyplot as plt

import conversationalspacemapapp.Types.Data as Data


class MapBarPlot:
    def __init__(
        self,
        fig: plt.figure,
    ):
        self.fig = fig
        self.ax = self.fig.gca()

    def plot(self, options: Data.PlotOptions):
        xlim_num = 0
        for utterance in options.map:
            self.ax.barh(
                utterance.number,
                utterance.words
                * options.get_participant_type(utterance.speaker).constant,
                align="center",
                height=0.8,
                color=options.get_participant_color(utterance.speaker),
                label=options.get_participant_label(utterance.speaker),
            )
            xlim_num = max([abs(utterance.words) for utterance in options.map]) * 1.1
        index = [*range(1, len(options.map) + 1)]

        # Set x-axis
        self.ax.set_xlim([-xlim_num, xlim_num])
        if not options.xaxis:
            self.ax.set(xticklabels=[])
            self.ax.tick_params(bottom=False)

        # Set grid
        if options.grid:
            self.ax.xaxis.grid(
                True, linestyle="--", which="major", color="grey", alpha=0.25
            )

        # Set y-axis
        self.ax.set_ylim([-2, max(index) + 2])
        if options.yaxis:
            self.ax.set_yticks(index)
            self.ax.set_ylabel("Utterance (bottom = start of interview)")
        else:
            self.ax.set(yticklabels=[])
            self.ax.tick_params(left=False)

        # Set plot labels
        if options.show_title:
            self.ax.set_title(options.title)
        if options.labels:
            self.ax.text(
                xlim_num / 2,
                -1,
                options.interviewee_label + "'s words per utterance",
                horizontalalignment="center",
            )
            self.ax.text(
                -xlim_num / 2,
                -1,
                options.interviewer_label + "'s words per utterance",
                horizontalalignment="center",
            )
        if options.legend:
            self.ax.legend(loc="upper left")
            self.remove_duplicate_labels_legend()

    def remove_duplicate_labels_legend(self):
        handles, labels = self.ax.get_legend_handles_labels()
        unique = [
            (h, l)
            for i, (h, l) in enumerate(zip(handles, labels))
            if l not in labels[:i]
        ]
        self.ax.legend(*zip(*unique))

    def save(self, filename: str):
        self.fig.savefig(filename, dpi=300)



import abc
import pathlib

import conversationalspacemapapp.Types.Data as Data


class AbstractParser(abc.ABC):
    """
    Converts aTrain transcripts into a dictionary containing the data for a conversational space map.
    """

    def __init__(self, file: pathlib.Path) -> None:
        self._file = file
        self._content = self._read_file()
        self._map: [Data.Utterance] = self._convert_text()

    @property
    def map(self) -> [Data.Utterance]:
        return sorted(self._map)

    @property
    def content(self) -> str:
        return self._content

    @property
    def participants(self) -> list:
        return sorted(list(set([utterance.speaker for utterance in self._map])))

    @abc.abstractmethod
    def number_of_words_by_speaker(self) -> [int, int]:
        raise NotImplementedError

    def _read_file(self) -> str:
        return self._file.read_text()

    def _clean_transcript(self) -> str:
        return self._content

    @abc.abstractmethod
    def _convert_text(self) -> dict[int:dict]:
        raise NotImplementedError

    @property
    def map_list(self) -> list[int]:
        """
        Return lists of words by utterance by speaker (only applies for two speakers), whereas the first speaker is the
        interviewer and the second speaker is the interviewee.
        """
        output = []
        for utterance in self._map:
            if utterance.number % 2 == 1:
                output.append(-utterance.words)
            else:
                output.append(utterance.words)
        return output



from mockito import when
from unittest.mock import MagicMock

import conversationalspacemapapp.Types.Data as Data
import conversationalspacemapapp.Types.Constants as Constants
import conversationalspacemapapp.App.AbstractApp as AbstractApp
import conversationalspacemapapp.Plotter.StylePicker as StylePicker
import conversationalspacemapapp.Parser.AbstractParser as AbstractParser


speaker00_name = "SPEAKER_00"
speaker00_label = "Label_00"
speaker00_type = Constants.Participant.Interviewer
speaker00_color = StylePicker.ColorPicker.pastel()[0]
speaker01_name = "SPEAKER_01"
speaker01_label = "Label_01"
speaker01_type = Constants.Participant.Interviewee
speaker01_color = StylePicker.ColorPicker.pastel()[1]


def get_parser_mock() -> AbstractParser.AbstractParser:
    parser = MagicMock()
    parser.map = [
        Data.Utterance(number=1, speaker=speaker00_name, words=10),
        Data.Utterance(number=2, speaker=speaker01_name, words=15),
    ]
    parser.participants = [speaker00_name, speaker01_name]
    return parser


def get_app_mock(
    parser: AbstractParser.AbstractParser = None,
) -> AbstractApp.AbstractApp:
    app = MagicMock()
    if parser is not None:
        app.parser = parser
    when(app)._get_participant_role(speaker00_name).thenReturn(speaker00_type)
    when(app)._get_participant_role(speaker01_name).thenReturn(speaker01_type)
    when(app)._get_participant_color(speaker00_name).thenReturn(speaker00_color)
    when(app)._get_participant_color(speaker01_name).thenReturn(speaker01_color)
    when(app)._get_participant_name(speaker00_name).thenReturn(speaker00_label)
    when(app)._get_participant_name(speaker01_name).thenReturn(speaker01_label)
    return app



import icnsutil

# Load the PNG file
file = "./../App/resources/icon.png"
output = ""

# compose
img = icnsutil.IcnsFile()
img.add_media(file=file)
img.write("./../App/resources/icon.ico")
img.write("./../App/resources/icon.icns")



import abc
import pathlib
from typing import Callable

import conversationalspacemapapp.Types.Constants as Constants
import conversationalspacemapapp.Parser.TimestampParser as TranscriptParser


class AbstractApp(metaclass=abc.ABCMeta):
    """
    Generate conversational space maps for interview data.
    """

    save_file_formats = ["PDF", "PNG", "SVG"]

    def __init__(self, **kwargs):
        self.parser: TranscriptParser.AbstractParser = None
        self.map = None
        super(AbstractApp, self).__init__(**kwargs)

    @property
    def path(self) -> pathlib.Path | None:
        return pathlib.Path(__file__)

    def path_as_str(self):
        if self.has_path:
            return self.path.as_posix()
        else:
            return ""

    @property
    def has_path(self):
        return self.path is not None

    @property
    def has_parser(self):
        return self.parser is not None

    def _create_window(self):
        home = self._create_home_layout()
        transcript = self._create_transcript_layout()
        about = self._create_about_layout()
        tab_menu = self._create_tab_menu(
            tabs=[["Home", home], ["Transcript", transcript], ["About", about]]
        )
        self._set_window(tab_menu)

    @abc.abstractmethod
    def _set_window(self, tab_menu):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_tab_menu(self, tabs):
        raise NotImplementedError

    def _create_home_layout(self):
        plot_settings = self._create_plot_settings_layout()
        participants = self._create_initial_participants_layout()
        label = self._create_info_layout()
        chart = self._create_chart()
        return self._set_home_window(plot_settings, participants, label, chart)

    @abc.abstractmethod
    def _set_home_window(self, plot_settings, participants, label, chart):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_plot_settings_layout(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_initial_participants_layout(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_participants_layout(self):
        assert self.has_path

    @abc.abstractmethod
    def _create_participant_layout(self, participant: str):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_info_layout(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _set_info_layout(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_chart(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_transcript_layout(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _set_transcript(self, content: str):
        raise NotImplementedError

    @abc.abstractmethod
    def _create_about_layout(self):
        raise NotImplementedError

    @abc.abstractmethod
    def draw_chart(self, chart, figure, *args, **kwargs):
        raise NotImplementedError

    @abc.abstractmethod
    def _button_factory(
        self,
        label: str,
        on_press: Callable,
        enabled: bool = True,
        padding: int = None,
        flex: int = None,
    ):
        raise NotImplementedError

    @abc.abstractmethod
    def _set_transparent_background(self):
        raise NotImplementedError

    async def open_handler(self, widget):
        path = await self._get_path()
        if path is not None and path.is_file():
            self._set_path(path)
        else:
            return

    @abc.abstractmethod
    async def _get_path(self) -> pathlib.Path | None:
        raise NotImplementedError

    @abc.abstractmethod
    def _set_path(self, path: pathlib.Path):
        raise NotImplementedError

    def _write_file_history(self):
        assert self.has_path
        history = pathlib.Path(__file__).parent / "assets" / "history.txt"
        content = self.path_as_str()
        if not self._is_new_history_path():
            return
        elif history.is_file():
            with open(history, "a") as f:
                f.write(content + "\n")
        else:
            history.parent.mkdir(exist_ok=True, parents=True)
            with open(history, "w") as f:
                f.write(content + "\n")

    def plot_handler(self, widget):
        assert self.has_parser
        self._update_plot()
        self._set_info_layout()

    @abc.abstractmethod
    def _update_plot(self):
        raise NotImplementedError

    async def save_handler(self, widget):
        assert self.has_parser
        path = await self._get_save_path()
        if path is not None:
            self.map.save(path)
        else:
            return

    @abc.abstractmethod
    async def _get_save_path(self) -> pathlib.Path | None:
        raise NotImplementedError

    def _get_info_content(self):
        assert self.has_parser
        total_words = sum(utterance.words for utterance in self.parser.map)
        text = f"Total words: {total_words} / "
        for participant in self.parser.participants:
            participant_words = sum(
                utterance.words if participant == utterance.speaker else 0
                for utterance in self.parser.map
            )
            text += f"Words {participant}: {participant_words} ({round(100 * participant_words / total_words, ndigits=1)}%) / "
        text += f"Total utterances: {len(self.parser.map)}"
        return text

    @abc.abstractmethod
    def _set_plot_title(self):
        raise NotImplementedError

    def _set_parser(self, widget=None):
        assert self.has_path
        self.parser = TranscriptParser.TimestampParser(self.path)
        self._create_participants_layout()
        self._set_plot_title()
        self._update_plot()
        self._set_info_layout()
        self._set_transcript(self.parser.content)
        self._write_file_history()

    @staticmethod
    def _get_file_history() -> [pathlib.Path]:
        history = pathlib.Path(__file__).parent / "assets" / "history.txt"
        if history.is_file():
            with open(history, "r") as f:
                output = []
                files = sorted(f.readlines())
                for file in files:
                    file = file.strip("\n")
                    file = pathlib.Path(file)
                    if file.is_file() and not file in output:
                        output.append(file)
                return output
        else:
            return []

    @abc.abstractmethod
    def _is_new_history_path(self) -> bool:
        raise NotImplementedError

    @abc.abstractmethod
    def _get_widget_value_by_id(self, key: str, default_value=None):
        raise NotImplementedError

    def _get_participant_role(self, speaker_name: str) -> Constants.Participant:
        role_key = speaker_name + "_role"
        if speaker_name in self.parser.participants:
            return self._get_widget_value_by_id(
                role_key, default_value=Constants.Participant.Undefined
            )
        return Constants.Participant.Undefined

    def _get_participant_color(self, speaker_name: str) -> str | None:
        color_key = speaker_name + "_color"
        if speaker_name in self.parser.participants:
            return self._get_widget_value_by_id(color_key)
        return None

    def _get_participant_name(self, speaker_name: str) -> str | None:
        name_key = speaker_name + "_name"
        if speaker_name in self.parser.participants:
            return self._get_widget_value_by_id(name_key)
        return None



import unittest

import conversationalspacemapapp.Types.Data as Data
import conversationalspacemapapp.Parser.TimestampParser as TranscriptParser

import tests.TestResources as TestResources


class TestTimestampParser(unittest.TestCase):
    # Unlimited return of error messages
    maxDiff = None

    # Define parser
    parser: TranscriptParser.AbstractParser

    @classmethod
    def setUpParser(cls, short=True, multiple_speaker=False):
        if multiple_speaker:  # Long transcript with multiple speakers
            return TranscriptParser.TimestampParser(
                file=TestResources.path_multiple_speaker_transcript
            )
        elif short:
            return TranscriptParser.TimestampParser(
                file=TestResources.path_short_transcript
            )
        else:
            return TranscriptParser.TimestampParser(
                file=TestResources.path_long_transcript
            )

    def test_transcript_timestamp_short(self):
        parser = TestTimestampParser.setUpParser()
        self.assertEqual(parser.content, TestResources.content_short_transcript)

    def test_transcript_timestamp_map_short(self):
        parser = TestTimestampParser.setUpParser()
        sut: [Data.Utterance] = parser.map
        utterance1, utterance2 = sut[0], sut[1]

        # Test first utterance
        self.assertEqual(utterance1.number, 1)
        self.assertEqual(utterance1.speaker, "Interviewer")
        self.assertEqual(utterance1.words, 15)

        # Test second utterance
        self.assertEqual(utterance2.number, 2)
        self.assertEqual(utterance2.speaker, "Interviewee")
        self.assertEqual(utterance2.words, 4)

    def test_transcript_timestamp_map_short_list(self):
        parser = TestTimestampParser.setUpParser()
        self.assertEqual(parser.map_list, [-15, 4])

    def test_transcript_timestamp_map_long(self):
        parser = TestTimestampParser.setUpParser(short=False)
        sut: [Data.Utterance] = parser.map
        utterance1, utterance2, utterance3 = sut[0], sut[1], sut[2]

        # Test first utterance
        self.assertEqual(utterance1.number, 1)
        self.assertEqual(utterance1.speaker, "SPEAKER_00")
        self.assertEqual(utterance1.words, 26)

        # Test second utterance
        self.assertEqual(utterance2.number, 2)
        self.assertEqual(utterance2.speaker, "SPEAKER_01")
        self.assertEqual(utterance2.words, 31)

        # Test third utterance
        self.assertEqual(utterance3.number, 3)
        self.assertEqual(utterance3.speaker, "SPEAKER_00")
        self.assertEqual(utterance3.words, 2)

    def test_transcript_timestamp_map_long_list(self):
        parser = TestTimestampParser.setUpParser(short=False)
        self.assertEqual(parser.map_list, [-26, 31, -2])

    def test_transcript_timestamp_speaker_words_short(self):
        parser = TestTimestampParser.setUpParser()
        self.assertEqual(parser.number_of_words_by_speaker, [15, 4])

    def test_transcript_timestamp_speaker_words_long(self):
        parser = TestTimestampParser.setUpParser(short=False)
        self.assertEqual(parser.number_of_words_by_speaker, [28, 31])

    def test_transcript_timestamp_participants_short(self):
        parser = TestTimestampParser.setUpParser()
        self.assertEqual(parser.participants, ["Interviewee", "Interviewer"])

    def test_transcript_timestamp_participants_multiple_speakers(self):
        parser = TestTimestampParser.setUpParser(multiple_speaker=True)
        self.assertEqual(
            parser.participants, ["SPEAKER_00", "SPEAKER_01", "SPEAKER_02"]
        )



import unittest
import matplotlib

matplotlib.use("Agg")

import conversationalspacemapapp.App.TogaApp.app as TogaApp
import conversationalspacemapapp.App.AbstractApp as AbstractApp

import tests.TestResources as TestResources


class TestAbstractApp(unittest.TestCase):
    sut: AbstractApp.AbstractApp

    def setUp(self):
        TestAbstractApp.sut = TogaApp.main()
        TestAbstractApp.sut.startup()


class TestTogaApp(TestAbstractApp):
    def test_run(self):
        pass

    def test_plot(self):
        TestAbstractApp.sut._set_path(
            path=TestResources.path_multiple_speaker_transcript
        )



import re

import conversationalspacemapapp.Types.Data as Data
import conversationalspacemapapp.Parser.AbstractParser as AbstractParser


class TimestampParser(AbstractParser.AbstractParser):
    @property
    def number_of_words_by_speaker(self) -> [int, int]:
        return [
            abs(sum(self.map_list[::2])),
            abs(sum(self.map_list[1::2])),
        ]

    def _clean_transcript(self) -> str:
        cleaned_transcript = re.sub(r"\n\[.*?] ", "\n", self._content)
        cleaned_transcript = cleaned_transcript.replace("\n-", "")
        return cleaned_transcript

    def _convert_text(self) -> dict[int:dict]:
        self._content = self._clean_transcript()
        tokens = self._content.split("\n\n")
        output = []
        counter = 0
        for token in tokens:
            segments = token.split(" ")
            number_of_words = len(segments) - 1
            if number_of_words > 0:
                counter += 1
                output.append(
                    Data.Utterance(
                        number=counter, speaker=segments[0], words=number_of_words
                    )
                )
        return output
